@@ -17,7 +17,7 @@
 /*
  *
  * @author Mladen Turk
- * @version $Id: ssl.c 1442587 2013-02-05 13:49:48Z rjung $
+ * @version $Id: ssl.c 1649733 2015-01-06 04:42:24Z billbarker $
  */
 
 #include "tcn.h"
@@ -190,6 +190,14 @@ static const jint supported_ssl_opts = 0
      | SSL_OP_PKCS1_CHECK_2
 #endif
 
+#ifdef SSL_OP_NO_TLSv1_1
+     | SSL_OP_NO_TLSv1_1
+#endif
+
+#ifdef SSL_OP_NO_TLSv1_2
+     | SSL_OP_NO_TLSv1_2
+#endif
+
 #ifdef SSL_OP_SINGLE_DH_USE
      | SSL_OP_SINGLE_DH_USE
 #endif
@@ -221,20 +229,39 @@ static const jint supported_ssl_opts = 0
 
 static int ssl_tmp_key_init_rsa(int bits, int idx)
 {
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(OPENSSL_USE_DEPRECATED)
     if (!(SSL_temp_keys[idx] =
-          RSA_generate_key(bits, RSA_F4, NULL, NULL)))
+          RSA_generate_key(bits, RSA_F4, NULL, NULL))) {
+#ifdef OPENSSL_FIPS
+        /**
+         * With FIPS mode short RSA keys cannot be
+         * generated.
+         */
+        if (bits < 1024)
+            return 0;
+        else
+#endif
         return 1;
-    else
+    }
+    else {
         return 0;
+    }
+#else
+    return 0;
+#endif
 }
 
 static int ssl_tmp_key_init_dh(int bits, int idx)
 {
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(OPENSSL_USE_DEPRECATED)
     if (!(SSL_temp_keys[idx] =
           SSL_dh_get_tmp_param(bits)))
         return 1;
     else
         return 0;
+#else
+    return 0;
+#endif
 }
 
 
@@ -747,6 +774,19 @@ TCN_IMPLEMENT_CALL(void, SSL, randSet)(TCN_STDARGS, jstring file)
     TCN_FREE_CSTRING(file);
 }
 
+TCN_IMPLEMENT_CALL(jint, SSL, fipsModeGet)(TCN_STDARGS)
+{
+    UNREFERENCED(o);
+#ifdef OPENSSL_FIPS
+    return FIPS_mode();
+#else
+    /* FIPS is unavailable */
+    tcn_ThrowException(e, "FIPS was not available to tcnative at build time. You will need to re-build tcnative against an OpenSSL with FIPS.");
+
+    return 0;
+#endif
+}
+
 TCN_IMPLEMENT_CALL(jint, SSL, fipsModeSet)(TCN_STDARGS, jint mode)
 {
     int r = 0;
@@ -860,8 +900,9 @@ static int jbs_write(BIO *b, const char *in, int inl)
     if (b->init && in != NULL) {
         BIO_JAVA *j = (BIO_JAVA *)b->ptr;
         JNIEnv   *e = NULL;
-        jbyteArray jb = (*e)->NewByteArray(e, inl);
+        jbyteArray jb;
         tcn_get_java_env(&e);
+        jb = (*e)->NewByteArray(e, inl);
         if (!(*e)->ExceptionOccurred(e)) {
             (*e)->SetByteArrayRegion(e, jb, 0, inl, (jbyte *)in);
             ret = (*e)->CallIntMethod(e, j->cb.obj,
@@ -879,8 +920,9 @@ static int jbs_read(BIO *b, char *out, int outl)
     if (b->init && out != NULL) {
         BIO_JAVA *j = (BIO_JAVA *)b->ptr;
         JNIEnv   *e = NULL;
-        jbyteArray jb = (*e)->NewByteArray(e, outl);
+        jbyteArray jb;
         tcn_get_java_env(&e);
+        jb = (*e)->NewByteArray(e, outl);
         if (!(*e)->ExceptionOccurred(e)) {
             ret = (*e)->CallIntMethod(e, j->cb.obj,
                                       j->cb.mid[1], jb);
@@ -970,7 +1012,7 @@ TCN_IMPLEMENT_CALL(jlong, SSL, newBIO)(TCN_STDARGS, jlong pool,
         goto init_failed;
     }
     j = (BIO_JAVA *)bio->ptr;
-    if ((j = (BIO_JAVA *)bio->ptr) == NULL) {
+    if (j == NULL) {
         tcn_ThrowException(e, "Create BIO failed");
         goto init_failed;
     }
@@ -1141,11 +1183,18 @@ TCN_IMPLEMENT_CALL(void, SSL, randSet)(TCN_STDARGS, jstring file)
     UNREFERENCED(file);
 }
 
+TCN_IMPLEMENT_CALL(jint, SSL, fipsModeGet)(TCN_STDARGS)
+{
+    UNREFERENCED(o);
+    tcn_ThrowException(e, "FIPS was not available to tcnative at build time. You will need to re-build tcnative against an OpenSSL with FIPS.");
+    return 0;
+}
+
 TCN_IMPLEMENT_CALL(jint, SSL, fipsModeSet)(TCN_STDARGS, jint mode)
 {
-    UNREFERENCED_STDARGS;
+    UNREFERENCED(o);
     UNREFERENCED(mode);
-
+    tcn_ThrowException(e, "FIPS was not available to tcnative at build time. You will need to re-build tcnative against an OpenSSL with FIPS.");
     return 0;
 }
 
